# Blobbit/Gloopy, a platformer video game
# Copyright (C) 2007  Christopher Allan Webber and Jeff Gibson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from worldsprite import WorldSprite
import copy


class Controllable(WorldSprite):
  def __init__(self, lower_x, lower_y, engine, image):
    WorldSprite.__init__(self, lower_x, lower_y, engine, image)

    self.is_controlled = False
    self.move_speed = .3
    self.jump_speed = .4
    self.jump_distance = 150 # pixels
    self.distance_jumped = 0
    self.jump_height = None
    self.fall_speed = .5
    self.is_jumping = False
    self.has_gravity = True
    self.facing = 'left'
    self.movement_queue = []

  def update(self):
    WorldSprite.update(self)

    if self.is_controlled and self.can_move():
      if self.engine.input.get('left'):
        self.move_left()
      if self.engine.input.get('right'):
        self.move_right()
      if self.engine.input.get('action1'):
        self.move_action1()
      if self.engine.input.get('action2'):
        self.move_action2()
      if not self.is_falling():
        if self.engine.input.get('jump') or self.is_jumping:
          self.move_jump()

    if self.has_gravity:
      self.move_fall()

    if self.movement_queue:
      self.safe_move()

  def can_move(self):
    if self.animation_state is None:
      return True
    else:
      if self.animations[self.animation_state][-1]['moveable']:
        return True

    return False

  def safe_move(self, safe=False):
    # Safely move the player based on direction
    # an attempt to abstract movement

    x, y = 0, 0
    for move in self.movement_queue:
      x += move[0]
      y += move[1]

      
    collisions, good_coordinates = self.move_collisions(
        self.engine.tiles, (x, y))

    # TODO: this checking is horrible make sure your levels have boundaries
    new_rect = self.rect.move(good_coordinates)
    if new_rect.left < 0 or \
        new_rect.left > self.engine.level.rightmost_pixel or \
        new_rect.bottom > self.engine.level.bottom_pixel:
          return 'out of bounds'


    if not safe:
      if self.is_monster:
        if self.is_controlled:
          self.engine.player.sprites()[0].rect.move_ip(good_coordinates)
      self.colliderect.move_ip(good_coordinates)
      self.rect.move_ip(good_coordinates)

    self.movement_queue = []

    return collisions


  def move_collisions(self, group, coordinates):
    # Returns:
    # crashed: [a list of sprites which collided during the move]
    # goodxy: (the closest the sprite could move without colliding)
    x, y = coordinates

    # Find out which direction we should be moving
    if x >= 0:
      x_step = 1
    else:
      x_step = -1

    if y >= 0:
      y_step = 1
    else:
      y_step = -1

    moved_rect = copy.copy(self.colliderect)
    x_crashed, y_crashed = [], []
    safe_x, safe_y = 0, 0
    # move one pixel at a time and see if we would hit something
    count = 1
    while count <= abs(x) or count <= abs(y):
      if not x_crashed and count <= abs(x):
        moved_rect.move_ip(x_step, 0)
        for s in group.sprites():
          if moved_rect.colliderect(s.colliderect):
            moved_rect.move_ip(-x_step, 0)
            if s not in x_crashed:
              x_crashed.append(s)
        if not x_crashed:
          safe_x += x_step

      if not y_crashed and count <= abs(y):
        moved_rect.move_ip(0, y_step)
        for s in group.sprites():
          if moved_rect.colliderect(s.colliderect):
            moved_rect.move_ip(0, -y_step)
            if s not in y_crashed:
              y_crashed.append(s)
        if not y_crashed:
          safe_y += y_step

      count += 1

    return set(x_crashed + y_crashed), (safe_x, safe_y)

  ##
  ## Individual movement methods which can be overridden in each subclass
  ##

  def is_falling(self):
    if not self.is_jumping:
      collisions, move = self.move_collisions(self.engine.tiles, (0, 1))

      if collisions:
        return False
      else:
        return True
    return False

  def move_jump(self):
    jump_speed = self.jump_speed
      
    if float(self.distance_jumped) / float(self.jump_distance) >= 0.7:
      jump_speed = self.jump_speed / 1.25
    if float(self.distance_jumped) / float(self.jump_distance) >= 0.9:
      jump_speed = self.jump_speed / 2
    speed = jump_speed * self.engine.elapsed

    if speed < 1:
      speed = 1

    if not self.jump_height:
      self.jump_height = self.rect.bottom - self.jump_distance
      self.is_jumping = True

    # see if we hit a ceiling
    collisions, _ = self.move_collisions(self.engine.tiles, (0, -1))
    
    if collisions or self.rect.bottom <= self.jump_height:
      self.stop_jump()
    else:
      self.movement_queue.append((0, -speed))
      self.distance_jumped += speed

  def stop_jump(self):
    self.jump_height = None
    self.distance_jumped = 0
    self.is_jumping = False


  def move_fall(self):
    if self.is_falling():
      speed = self.fall_speed * self.engine.elapsed
      self.distance_jumped = int(speed)

      self.movement_queue.append((0, speed))

  def move_right(self):
    self.facing = 'right'
    if not self.animation_state:
      if not self.is_monster:
        self.set_animation('walking')
      else:
        if self.is_controlled:
          self.set_animation('walking_infested')
        else:
          self.set_animation('walking')
    speed = self.move_speed * self.engine.elapsed

    self.movement_queue.append((speed, 0))

  def move_left(self):
    self.facing = 'left'
    if not self.animation_state:
      if not self.is_monster:
        self.set_animation('walking')
      else:
        if self.is_controlled:
          self.set_animation('walking_infested')
        else:
          self.set_animation('walking')
    speed = self.move_speed * self.engine.elapsed

    self.movement_queue.append((-speed, 0))

  def move_action1(self):
    pass

  def move_action2(self):
    pass
