# Blobbit/Gloopy, a platformer video game
# Copyright (C) 2007  Christopher Allan Webber and Jeff Gibson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pygame
import os
import copy
import camera
import player
    
import levelparse
import worldsprite

from pygame import *

RESOLUTION = (800, 600)

class Engine(object):
  def __init__(self, directory):
    self.counter = 0
    pygame.init()
    self.screen = pygame.display.set_mode(RESOLUTION, pygame.HWACCEL)
    self.clock = pygame.time.Clock()

    self.camera = camera.Camera(self, RESOLUTION[0], RESOLUTION[1] - 32, 0, 0)
    self.directory = directory
    self.images = {}
    self.__load_all_images()
    self.__load_all_sounds()

    self.input = {}
    self.level = Level()

    # We want some initial number, anyway
    self.elapsed = 0

    # TODO: get this music out of engine
    music = os.path.join(self.directory, 'data/sound/gloopy_music1.mp3')
    pygame.mixer.music.load(music)
    pygame.mixer.music.play(-1)


    self.tiles = pygame.sprite.Group()
    self.monsters = pygame.sprite.Group()
    self.projectiles = pygame.sprite.Group()
    self.items = pygame.sprite.Group()
    self.player = pygame.sprite.GroupSingle()

  def load_image(self, name, filename, convert_alpha=False,
                 xflip=False, yflip=False):
    if convert_alpha:
      self.images[name] = pygame.image.load(filename).convert_alpha()
    else:
      self.images[name] = pygame.image.load(filename).convert()
    if xflip or yflip:
      self.images[name] = pygame.transform.flip(self.images[name],
                                                xflip,
                                                yflip)


  # TODO: get this function out of the engine
  def __load_all_images(self):
    # specify files that should be loaded without transparency
    no_transparency = ('metal', 'ice', 'pipe_background')

    convert_alpha = lambda x: x not in no_transparency

    path = os.path.join(self.directory, 'data/gfx')
    for file in os.listdir(path):
      name, ext = file.split('.')
      if ext == 'png':
        self.load_image(name, os.path.join(path, file), convert_alpha(name))
        self.load_image(name + "-flip", os.path.join(path, file),
                        convert_alpha(name), True)


  def __load_all_sounds(self):
    path = os.path.join(self.directory, 'data/sound')
    self.sounds = {}
    files = ['fireball', 'high_splat', 'low_splat', 'ice_break', 'fire_monster_dies']
    for file in files:
      fullname = file + '.ogg'
      filename = os.path.join(path, fullname)
      print filename
      sound = pygame.mixer.Sound(filename)
      self.sounds[file] = sound


  # TODO: get this function out of the engine
  def generate_background(self, image):
    image_rect = image.get_rect()
    self.background = pygame.sprite.Group()    
    for x in range(0, self.level.rightmost_pixel, image_rect.width):
      for y in range(0, self.level.bottom_pixel, image_rect.width):
        background_panel = worldsprite.WorldSprite(x, y, self, image)
        self.background.add(background_panel)


  def run(self):
    # TODO: break all of this level stuff out of the engine and into Level
    l = levelparse.LevelParse(self)
    self.tiles, self.monsters, self.items, self.player = l.Parse('level')
    l.rightmost_pixel = 10000
    
    background = pygame.surface.Surface(
        (self.camera.rect.width, self.camera.rect.height)).convert()
    background.fill((70, 70, 70))

    while True:
      self.elapsed = self.clock.tick(60)
      self.check_input()

      # logic updates
      self.monsters.update()
      self.player.update()
      self.items.update()
      self.projectiles.update()

      # update the camera's position, and shite
      self.camera.update()

      self.camera.camera_surface.blit(background, (0, 0))

      # draw stuff on camera
      self.camera.draw_group(self.tiles)
      self.camera.draw_group(self.items)
      self.camera.draw_group(self.monsters)
      self.camera.draw_group(self.player)
      self.camera.draw_group(self.projectiles)

      # blit camera and HUD to screen

      self.screen.blit(self.camera.camera_surface, (0,0))

      pygame.display.flip()
        
  def check_input(self):
    events = pygame.event.get()
    for event in events:
      if event.type == KEYDOWN:
        if event.key == K_UP:
          self.input['up'] = True
        if event.key == K_DOWN:
          self.input['down'] = True
        if event.key == K_LEFT:
          self.input['left'] = True
        if event.key == K_RIGHT:
          self.input['right'] = True
        if event.key == K_SPACE:
          self.input['jump'] = True
        if event.key == K_LCTRL:
          self.input['action1'] = True
        if event.key == K_LALT:
          self.input['action2'] = True
        if event.key == K_ESCAPE:
          # TODO: add a menu action here
          assert 0
      if event.type == KEYUP:
        if event.key == K_UP:
          self.input['up'] = False
        if event.key == K_DOWN:
          self.input['down'] = False
        if event.key == K_LEFT:
          self.input['left'] = False
        if event.key == K_RIGHT:
          self.input['right'] = False
        if event.key == K_SPACE:
          self.input['jump'] = False
        if event.key == K_LCTRL:
          self.input['action1'] = False
        if event.key == K_LALT:
          self.input['action2'] = False

class Level(object):
  def __init__(self):
    self.bottom_pixel = 0
    self.rightmost_pixel = 0

if __name__ == "__main__":
  a = Engine()
  a.run()
