# Blobbit/Gloopy, a platformer video game
# Copyright (C) 2007  Christopher Allan Webber and Jeff Gibson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from controllable import Controllable
import projectiles
import random
import copy

class Monster(Controllable):
  def __init__(self, lower_x, lower_y, engine, image):
    Controllable.__init__(self, lower_x, lower_y, engine, image)

    self.is_controlled = False
    self.is_monster = True

    # Blind as a bat!
    self.x_vision_range = 0
    self.y_vision_range = 0

    self.player_mass_min = 0
    self.player_mass_max = 0

    self.health = 0

  def take_damage(self, damage):
    self.health -= damage
    if self.health < 0:
      self.kill()
      self.engine.sounds['fire_monster_dies'].play()
      if self.is_controlled:
        self.give_control()

  def update(self):
    if self.is_controlled:
      if self.animation_state:
        if '_infested' not in self.animation_state:
          self.animation_state = self.animation_state + '_infested'
      self.engine.player.sprites()[0].colliderect = copy.copy(self.colliderect)

    if not self.is_controlled:
      self.update_ai()

    Controllable.update(self)

  def sees_player(self):
    player = self.engine.player.sprites()[0]
    xdiff = self.rect.centerx - player.rect.centerx
    ydiff = self.rect.bottom - player.rect.bottom
    if abs(xdiff) < self.x_vision_range and abs(ydiff) < self.y_vision_range:
      if xdiff >= 0 and self.facing == "left" or \
           xdiff <= 0 and self.facing == "right":
        return True
    return False

  def update_ai():
    pass

  def give_control(self):
    if self.is_controlled:
      player = self.engine.player.sprites()[0]
      self.is_controlled = False
      player.is_controlled = True
      player.is_visible = True
      player.rect.bottom = copy.copy(self.rect.top)
      player.colliderect = copy.copy(player.rect)
      player.move_jump()
      player.controlling_monster = None
      self.stop_jump()
      self.clear_animation()

  def move_action2(self):
    self.give_control()


class Drakko(Monster):
  def __init__(self, lower_x, lower_y, engine):
    Monster.__init__(self, lower_x, lower_y, 
        engine, engine.images['drakko'])

    self.move_speed = .2

    self.health = 20

    self.player_mass_min = 30
    self.animations = {
      'walking' : (
        (engine.images['drakko-walk1'], 200),
        (engine.images['drakko-walk2'], 200),
        (engine.images['drakko-walk3'], 200),
        (engine.images['drakko-walk4'], 200),
        {'repeats': False, 'moveable': True}
      ),
      'walking_infested' : (
        (engine.images['drakko-walk1_infested'], 200),
        (engine.images['drakko-walk2_infested'], 200),
        (engine.images['drakko-walk3_infested'], 200),
        (engine.images['drakko-walk4_infested'], 200),
        {'repeats': False, 'moveable': True}
      ),
      'shooting' : (
        (engine.images['drakko-fire_breathing'], 300),
        {'repeats': False, 'moveable': False}
      ),
      'shooting_infested' : (
        (engine.images['drakko-fire_breathing_infested'], 300),
        {'repeats': False, 'moveable': False}
      ),
    }

    self.colliderect.inflate_ip(-40, -15)
    self.colliderect.bottom = self.rect.bottom
    self.colliderect.centerx = self.rect.centerx
    self.colliderect.left += 5

    self.x_vision_range = 400
    self.y_vision_range = 300

    self.ai_fire_wait_min = 200
    self.ai_fire_wait_max = 2500
    self.ai_fire_waiting = 0

    self.facing = "right"


  def update(self):
    #if self.facing == 'right' and self.default_image is not 

    Monster.update(self)

    if self.is_controlled:
      if not self.animation_state:
        self.set_image(self.engine.images['drakko_infested'])

  ##
  ## Movement methods
  ##

  def fire_fireball(self, player_ball=False):
    if self.is_controlled:
      self.set_animation('shooting_infested')
    else:
      self.set_animation('shooting')

    flame_direction = self.facing
    if self.is_controlled:
      if self.engine.input.get('up'):
        flame_direction = 'up' + self.facing
      if self.engine.input.get('down'):
        flame_direction = 'down' + self.facing
      
    if player_ball:
      immune_sprites = ([self, self.engine.player.sprites()[0]])
    else:
      immune_sprites = (self.engine.monsters.sprites())

    if self.facing == "right":
      self.engine.sounds['fireball'].play()
      projectile = projectiles.FireBall(
        self.rect.centerx, self.rect.centery + 20, self.engine, flame_direction, immune_sprites)
    elif self.facing == "left":
      # Don't know why this makes it look better, but it does.
      projectile = projectiles.FireBall(
        self.rect.left, self.rect.centery + 20, self.engine, flame_direction, immune_sprites)
    self.engine.projectiles.add(projectile)

  def move_action1(self):
    self.fire_fireball(player_ball=True)

  def update_ai(self):
    self.ai_walk()

    ## Firing related stuffs
    if self.ai_fire_waiting:
      self.ai_fire_waiting -= self.engine.elapsed
      if self.ai_fire_waiting < 0:
        self.ai_fire_waiting = 0

    if self.sees_player() and not self.ai_fire_waiting:
      self.fire_fireball()
      self.ai_fire_waiting = random.randint(self.ai_fire_wait_min, self.ai_fire_wait_max)

  def reverse_direction(self):
    if self.facing == 'right':
      self.facing = 'left'
    else:
      self.facing = 'right'

  def ai_walk(self):
    if self.facing == 'right':
      speed = self.move_speed * self.engine.elapsed
      self.movement_queue.append((speed, 0))
      collisions = self.safe_move(safe=True)
      if not collisions:
        self.move_right()
      else:
        self.reverse_direction()
        self.move_left()
    else:
      speed = self.move_speed * self.engine.elapsed
      self.movement_queue.append((-speed, 0))
      collisions = self.safe_move(safe=True)
      if not collisions:
        self.move_left()
      else:
        self.reverse_direction()
        self.move_right()
