# Blobbit/Gloopy, a platformer video game
# Copyright (C) 2007  Christopher Allan Webber and Jeff Gibson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from controllable import Controllable
import pygame
import copy
import sys

class Blobbit(Controllable):
  def __init__(self, lower_x, lower_y, engine):
    Controllable.__init__(
        self, lower_x, lower_y, engine, engine.images['blobbit'])

    self.move_speed = .25

    self.animations = {}

    self.protected_animations = {
      'walking': (
          (engine.images['blobbit'], 150),
          (engine.images['blobbit-walking1'], 150),
          (engine.images['blobbit-walking2'], 150),
          (engine.images['blobbit-walking3'], 175),
          (engine.images['blobbit-walking2'], 150),
          (engine.images['blobbit-walking1'], 150),
          {'repeats': False, 'moveable': True}
      ),

      #flipped
      'walking-flip': (
          (engine.images['blobbit-flip'], 150),
          (engine.images['blobbit-walking1-flip'], 150),
          (engine.images['blobbit-walking2-flip'], 150),
          (engine.images['blobbit-walking3-flip'], 175),
          (engine.images['blobbit-walking2-flip'], 150),
          (engine.images['blobbit-walking1-flip'], 150),
          {'repeats': False, 'moveable': True}
      ),
    }

    self.protected_default_image = self.default_image

    self.has_gravity = True
    self.is_controlled = True
    self.engine.camera.controlling_sprite = self
    self.controlling_monster = None
    self.resize_blobbit(40)
    self.mass = 40
    self.image = self.default_image
    self.min_mass = 20 # FALL BELOW THIS AND DIE

  def take_damage(self, damage):
    self.mass -= damage
    if self.mass < self.min_mass:
      self.die()
      return False
    else:
      self.resize_blobbit(self.mass)

    if self.controlling_monster:
      if self.mass < self.controlling_monster.player_mass_min:
        self.controlling_monster.give_control()

  def increase_mass(self, mass):
    self.mass += mass
    difference = mass
    if self.mass > 100:
      difference = self.mass - 100
      self.mass = 100
    self.resize_blobbit(self.mass)

  def die(self):
    # Okay this is not ideal, but we can't do anything better within
    # the timeframe
    print "Blobbit... is dead."
    sys.exit()

  def resize_blobbit(self, percent):
    new_anims = {}

    for anim_name, frames in self.protected_animations.iteritems():
      this_new_anim = []
      for item in frames:
        if isinstance(item, dict):
          this_new_anim.append(item)
        else:
          image, time = item
          scaled_image = self.scale_image(image, percent)
          this_new_anim.append((scaled_image, time))
      new_anims[anim_name] = this_new_anim
    self.animations = new_anims

    # And now change default image
    scaled_default_image = self.scale_image(self.protected_default_image,
                                            percent)
    self.default_image = scaled_default_image
    current_bottom = self.rect.bottom
    current_centerx = self.rect.centerx
    self.rect = copy.copy(self.default_image.get_rect())
    self.rect.centerx = current_centerx
    self.rect.bottom = current_bottom
    if not self.controlling_monster:
      self.colliderect = copy.copy(self.rect)

  def scale_image(self, image, percent):
    original_rect = image.get_rect()
    new_width = int(original_rect.width * percent * .01)
    new_height = int(original_rect.height * percent * .01)
    scaled_image = pygame.transform.scale(image, (new_width, new_height))
    return scaled_image

  def update(self):
    Controllable.update(self)
    
    if self.is_controlled:
      self.check_attach()


  def check_attach(self):
    # TODO: maybe only check monsters on camera
    if self.is_falling():
      attach_rect = pygame.Rect(
        self.colliderect.left, 0, self.colliderect.width, 30)
      attach_rect.bottom = self.colliderect.bottom

      for monster in self.engine.monsters:
        monster_rect = pygame.Rect(
          monster.colliderect.left, 0, monster.colliderect.width, 10)
        monster_rect.top = monster.colliderect.top

        if attach_rect.colliderect(monster_rect):
          self.give_control(monster)
          break

  def give_control(self, sprite_to_control):
    if self.is_controlled and \
         self.mass >= sprite_to_control.player_mass_min:
      # We use drawings of blobbit drawn on top of the enemy for the
      # enemy itself, rather than blitting blobbit on top of that
      # enemy.  So hide Blobbit.
      self.is_controlled = False
      self.is_visible = False
      self.clear_animation()
      sprite_to_control.is_controlled = True
      self.controlling_monster = sprite_to_control
      self.colliderect = copy.copy(sprite_to_control.colliderect)
