# Blobbit/Gloopy, a platformer video game
# Copyright (C) 2007  Christopher Allan Webber and Jeff Gibson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from worldsprite import WorldSprite
import math

NORMAL_DIRECTIONS = ("up", "down", "left", "right")
DIAG_DIRECTIONS = ("upleft", "upright", "downleft", "downright")

DIAG_MOVE = math.sqrt(2)

class Error(Exception):
  pass

class NotADirection:
  pass

class Projectile(WorldSprite):
  def __init__(self, lower_x, lower_y, engine, image, direction, speed):
    WorldSprite.__init__(self, lower_x, lower_y, engine, image)

    self.direction = direction

    self.speed = speed
    
  def update(self):
    newx, newy = self.move()
    self.rect.x = newx
    self.rect.y = newy
    self.suicide_if_offscreen()

  def suicide_if_offscreen(self):
    if self.rect.left > self.engine.level.rightmost_pixel or \
         self.rect.right < 0 or \
         self.rect.bottom < 0 or \
         self.rect.top > self.engine.level.bottom_pixel:
      self.kill()
      return False

  def move(self):
    move = self.engine.elapsed * self.speed
    diagonal_move = move * .75
    
    newx = self.rect.x
    newy = self.rect.y
    if self.direction == "up":
      newy = self.rect.y - move
    elif self.direction == "down":
      newy = self.rect.y + move
    elif self.direction == "left":
      newx = self.rect.x - move
    elif self.direction == "right":
      newx = self.rect.x + move
    elif self.direction == "upleft":
      newx = self.rect.x - diagonal_move
      newy = self.rect.y - diagonal_move
    elif self.direction == "upright":
      newx = self.rect.x + diagonal_move
      newy = self.rect.y - diagonal_move
    elif self.direction == "downleft":
      newx = self.rect.x - diagonal_move
      newy = self.rect.y + diagonal_move
    elif self.direction == "downright":
      newx = self.rect.x + diagonal_move
      newy = self.rect.y + diagonal_move

    return newx, newy

FIREBALL_DIRECTION_MAP = {
  'right': 'fireball',
  'left': 'fireball-flip',
  'up': 'fireball-up',
  'down': 'fireball-down',
  'upright': 'fireball-tilt_up',
  'upleft': 'fireball-tilt_up-flip',
  'downright': 'fireball-tilt_down',
  'downleft': 'fireball-tilt_down-flip',
  }


class FireBall(Projectile):
  def __init__(self, lower_x, lower_y, engine, direction, immune_sprites):
    Projectile.__init__(self, lower_x, lower_y, engine,
                        engine.images[FIREBALL_DIRECTION_MAP[direction]],
                        direction, speed=.5)
    self.immune_sprites = immune_sprites
    self.damage_power = 5
  
  def update(self):
    Projectile.update(self)
    self.fry_tiles()
    self.hurt_critters()

  def fry_tiles(self):
    for tile in self.engine.tiles:
      if self.rect.colliderect(tile.rect):
        self.kill()
        if hasattr(tile, 'fry_self'):
          tile.fry_self()

  def hurt_critters(self):
    critters = self.engine.monsters.sprites()
    critters.extend(self.engine.player.sprites())

    for critter in critters:
      if critter not in self.immune_sprites:
        if self.rect.colliderect(critter.rect):
          self.kill()
          critter.take_damage(5)
