# Blobbit/Gloopy, a platformer video game
# Copyright (C) 2007  Christopher Allan Webber and Jeff Gibson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pygame.transform import flip
from pygame.sprite import Sprite
from pygame.rect import Rect
import copy

class WorldSprite(Sprite):
  def __init__(self, lower_x, lower_y, engine, image):
    Sprite.__init__(self)

    self.engine = engine

    self.image = image
    if self.image:
      self.rect = image.get_rect()
    else:
      self.rect = Rect()
    self.rect.bottom = lower_y
    self.rect.left = lower_x
    self.colliderect = copy.copy(self.rect)

    self.default_image = self.image

    self.is_monster = False

    # Some notes on animation:

    # By default, an object inheriting from WorldSprite will not be animated,
    # have no animations in its animations dictionary, and have other sensible
    # defaults.

    # By naming an animation 'default' in its animations dict however, you 
    # are telling WorldSprite that that's the animation you want on all the time
    # whenever another animation isn't happening.  If this ever isn't your desired
    # behaviour, we have a secondary "kill switch" called 'kill_animation'.

    # WorldSprite's update method will obey this above all, so be careful with it.

    self.is_animated = False
    self.animations = {}
    self.animation_state = None
    self.animation_frame = 0
    self.animation_timer = 0
    self.animation_repeats = False
    self.kill_animation = False
    self.walk_speed = .15
    self.fall_speed = 1
    self.has_gravity = False

    # If this is False then, well, obviously we won't see the sprite.
    self.is_visible = True

  def relative_rect(self, camera):
    return Rect(self.rect.x - camera.rect.x,
                self.rect.y - camera.rect.y,
                self.rect.width,
                self.rect.height)

  def update(self):
    # check to see if there is a default animation for this object

    if not self.is_animated:
      if self.animations.get('default'):
        self.set_animation('default')

    # only update animation if it's animated, and the kill switch isn't on
    if self.is_animated:
      if not self.kill_animation:
        self.update_animation()

  def update_animation(self):
    self.animation_timer += self.engine.elapsed
    frame_image, frame_timer = \
        self.animations[self.animation_state][self.animation_frame]
    # check to see if the animation timer has elapsed 
    if self.animation_timer >= frame_timer:
      # subtract the number of ms it should have used so we don't run
      # faster than we should
      self.animation_timer -= frame_timer
      # and increase the frame by 1
      self.animation_frame += 1
      # if we've looped through an animation we'll see if it should repeat
      if self.animation_frame == len(self.animations[self.animation_state]) - 1:
        if self.animation_repeats: 
          self.animation_frame = 0
          self.set_image(self.animations[self.animation_state][0][0])
        else:
          # if it doesn't repeat, but has a default animation, we'll switch
          # to it
          if self.animations.get('default'):
            self.set_animation('default')
          else:
            # otherwise clear the animation
            self.clear_animation()
      else:
        # if we haven't looped through, set the new image for the current frame
        self.set_image(
                  self.animations[self.animation_state][self.animation_frame][0])


  def set_image(self, frame):
    if self.facing == 'left':
      self.image = self.flip_image(frame)
    else:
      self.image = frame
    rect = self.image.get_rect()
    self.rect.width = rect.width
    self.rect.height = rect.height


  def flip_image(self, image):
    return flip(image, True, False)


  def set_animation(self, animation_name, flip=False):
    # make sure the animation isn't already running
    if self.animation_state != animation_name:
      if self.animations.get(animation_name):
        self.is_animated = True
        self.animation_repeats = self.animations[animation_name][-1]['repeats']
        self.animation_state = animation_name
        self.animation_frame = 0
        self.animation_timer = 0

        # set the image to the first frame of the current animation
        animation = self.animations[animation_name]
        self.set_image(self.animations[animation_name][0][0])

  def clear_animation(self):
    self.is_animated = False
    self.animation_state = None
    self.set_image(self.default_image)
